#!/usr/bin/perl
require ('/var/www/virtuals/old.gorod.ua/html/data/uri/clients_conf.pl');
require('/var/www/virtuals/old.gorod.ua/html/data/uri/clients_conf_rev.pl');
use Tie::IxHash;
use POSIX;
use List::Util qw[min max];

=pod
1  - schema
2  - subdomain
3  - domain of subdomain (gorod.ua,gorod.dn.ua,ukr-info.net)
4  - personal domain
5  - ID from prefix (data/ID)
6  - base+path+args
7  - base+path
8  - base (/price,/press...)
9  - path
10 - args
=cut

tie %ur_vars, 'Tie::IxHash';
%ur_vars=(
	'capture'=>qr/^(?:(https?):\/\/(?:www\.)?)?(?:(?:([-a-z0-9.]+\.)?(gorod(?:\.dn)?\.ua)|(?:ukr-info\.net))|([^\/]+?))?(?:\/data\/(\d+?))?(((\/[^\/\.\?]+?)?(?:\/([^\?]+?))?)(?:\?(.+?))?)(?:\/|\/index\.(?:html?|cgi|pl))?$/,
	'directMap'=>{
		'/'=>['/data/pages.pl?id=${id}'],
		'/price'=>[qr/[^\d\/]/,'/data/price.pl?id=${id}&page=${page}&words=${words}','/data/price.pl?id=${id}&vs=${arg[0]}'],
		'/press'=>[qr/[^\d\/]/,'/data/press.pl?id=${id}&page=${page}','/data/press.pl?id=${id}&aid=${arg[0]}'],
		'/vakansii'=>[qr/[^\d\/]/,'/data/vakansii.pl?id=${id}&page=${page}','/data/vakansii.pl?id=${id}&vid=${arg[0]}'],
		'/resume'=>[qr/[^\d\/]/,'/data/resume.pl?id=${id}&page=${page}','/data/resume.pl?id=${id}&rid=${arg[0]}'],
		'/dbprice'=>[qr/[^\d\/]/,'/data/dbprice/price.cgi?id=${id}','/data/dbprice/price.cgi?id=${id}&cc=${arg[0]}','/data/dbprice/info.cgi?id=${id}&cc=${arg[0]}&gid=${arg[1]}'],
		'/contents'=>[qr/(*F)/,'/data/contents/${id}/${path}'],
		'/map'=>[qr/(*F)/,'/include/maps/frame_map.php?idms=${id}']
	},
	'reverseMap'=>{
		'/price.pl'=>'/price${vs}',
		'/press.pl'=>'/press${aid}',
		'/vakansii.pl'=>'/vakansii${vid}',
		'/resume.pl'=>'/resume${rid}',
		'/dbprice/price.cgi'=>'/dbprice${cc}',
		'/dbprice/info.cgi'=>'/dbprice${cc}${gid}',
	}
);
sub ur_direct {
	local ($type,$path,$args,$id,$passThrough)=@_;
	local ($page);
	$type=$type||'/';
	if((my @map=@{$ur_vars{'directMap'}{$type}})) {
		if(!($path=~$map[0])) {
			local @arg=split('/',$path);
			$args=~/(?:\?|&|^)(page|words)=([^&?]+)(?{${$1}=$2;})(*F)/;
			my $line=$map[min(scalar @map-1,scalar @arg+1)];
			$line=~s/\$\{(\w+)(?:\[([\d.]+)\])?\}/defined $2?${$1}[$2]:${$1}/ge;
			return $line;
		}
	}
	return ($passThrough?$uri:'');
}

sub ur_reverse {
	local ($path,$args,$id,$passThrough)=@_;
	local ($cc,$gid,$vid,$ris,$aid,$vis);
	if(my $line=$ur_vars{'reverseMap'}{$path}) {
		$args=~/(?:\?|&|^)(vs|aid|vid|rid|cc|gid)=([^&?]++)(?{${$1}=$2;})(*F)/;
		$line=~s/\$\{(\w+)\}/\/${$1}/g;
		return $line;
	} else {
		return ($passThrough?$uri:'');
	}
}

sub ur_link_rewrite {
	if($_[0]=~$ur_vars{'capture'}) {
		my $domain=($2||$4),$uri,$base=$3;
		if($domain) {
			if(!$6||$6 eq '/') {
				$domain=~s/\.$/.$base/;
				return "http://$domain";
			} elsif (my $id=$client_sites_rev{$domain}) {
				return (($id != $cid)?('http://'.$client_sites{$id}[0]):'').ur_reverse($7,$10,$id||$5||$cid)||$6||'/';
			} else {	
				return $_[0];
			}
		} else {
			if($8 eq '/include') {
				return ((!$client_sites{$cid}[0])?"/data":'').$6;
			} else {
				return ((!$client_sites{$cid}[0])?"/data/$cid":'').ur_reverse($7,$10,$5||$cid)||$6||'/';
			}
		}
	} else {
		return $_[0];
	}
}

sub ur_process(){
	for my $val (@_) {
		$val=~s/(<(?:a|img|link|script).*?(?:href|src)=")([^"]+)/$1.ur_link_rewrite($2)/ge;
	}
}

sub ur_a{
	my ($uri,%params,$id)=($_[0],%{@_[1]},$_[2]);
	$id=$id||$cid;
	if(%params) {
		my @args=();
		for my $key (keys %params) {
			if(my $val=$params{$key}) {
				push(@args,"$key=$val");
			}
		}
		$args=scalar @args?('?'.join('&',@args)):'';
	}
	return (!$client_sites{$id}[0])?"/data/$id$_[0]$args":"$_[0]$args";
}

1;
